@@ -4,7 +4,10 @@
 import asyncio
 import binascii
 import logging
+import time
+import platform
 
+import bleak
 from bleak import discover
 from bleak import BleakClient
 
@@ -17,15 +20,39 @@ logger = logging.getLogger(__name__)
 
 
 class Scanner:
+    UUIDS = [
+        "0000180a-0000-1000-8000-00805f9b34fb",
+        "00010203-0405-0607-0809-0a0b0c0d1912",
+        "0000fe00-0000-1000-8000-00805f9b34fb",
+    ]
+
+    BLEAK_KWARGS = {
+        "service_uuids": UUIDS
+    }
+
     def __init__(self):
         self.devices_dict = {}
         self.devices_list = []
         self.receive_data = []
         self.walking_belt_candidates = []  # type: list[BLEDevice]
 
-    async def scan(self):
+    @staticmethod
+    def is_darwin():
+        try:
+            return platform.system().lower() == 'darwin'
+        except:
+            return False
+
+    @staticmethod
+    def get_bleak_kwargs():
+        return Scanner.BLEAK_KWARGS if Scanner.is_darwin() else {}
+
+    async def scan(self, timeout=3.0):
+        kwargs = Scanner.get_bleak_kwargs()
         logger.info("Scanning for peripherals...")
-        dev = await discover()
+        logger.debug("Scanning kwargs: %s" % (kwargs,))
+        scanner = bleak.BleakScanner(**kwargs)
+        dev = await scanner.discover(timeout=timeout, **kwargs)
         for i in range(len(dev)):
             # Print the devices discovered
             info_str = ', '.join(["[%2d]" % i, str(dev[i].address), str(dev[i].name), str(dev[i].metadata["uuids"])])
@@ -104,6 +131,7 @@ class WalkingPadCurStatus:
         self.app_speed = 0
         self.belt_state = 0
         self.manual_mode = 0
+        self.rtime = 0
 
     def load_from(self, cmd):
         self.raw = bytearray(cmd)
@@ -115,6 +143,7 @@ class WalkingPadCurStatus:
         self.steps = WalkingPad.byte2int(cmd[11:])
         self.app_speed = cmd[14]  # / 30
         self.controller_button = cmd[16]
+        self.rtime = time.time()
 
     @staticmethod
     def check_type(cmd):
@@ -142,12 +171,14 @@ class WalkingPadLastStatus:
         self.dist = 0
         self.time = 0
         self.steps = 0
+        self.rtime = 0
 
     def load_from(self, cmd):
         self.raw = bytearray(cmd)
         self.time = WalkingPad.byte2int(cmd[8:])
         self.dist = WalkingPad.byte2int(cmd[11:])
         self.steps = WalkingPad.byte2int(cmd[14:])
+        self.rtime = time.time()
 
     @staticmethod
     def check_type(cmd):
@@ -176,8 +207,10 @@ class Controller:
         self.char_fe02 = None
         self.client = None
         self.last_raw_cmd = None
+        self.last_cmd_time = None
         self.last_status = None
         self.last_record = None
+        self.minimal_cmd_space = 0.69
 
         self.handler_cur_status = None
         self.handler_last_status = None
@@ -238,14 +271,22 @@ class Controller:
         if not address:
             raise ValueError('No address given to connect to')
 
-        self.client = BleakClient(address)
-        return await self.client.connect()
+        logger.info("Connecting to %s" % (address,))
+        kwargs = Scanner.get_bleak_kwargs()
+        self.client = BleakClient(address, **kwargs)
+        return await self.client.connect(timeout=10.0, **kwargs)
 
     async def send_cmd(self, cmd):
         self.fix_crc(cmd)
+        if self.last_cmd_time and time.time() - self.last_cmd_time < self.minimal_cmd_space:
+            to_sleep = max(0, min(time.time() - self.last_cmd_time, self.minimal_cmd_space))
+            await asyncio.sleep(to_sleep)
+
         return await self.send_cmd_raw(cmd)
 
     async def send_cmd_raw(self, cmd):
+        self.last_raw_cmd = cmd
+        self.last_cmd_time = time.time()
         r = await self.client.write_gatt_char(self.char_fe02, cmd)
         return r
 
@@ -316,7 +357,7 @@ class Controller:
         await self.connect(address)
         client = self.client
 
-        x = await client.is_connected()
+        x = client.is_connected
         logger.info("Connected: {0}".format(x))
 
         self.char_fe01 = None
